@@ -60,7 +60,7 @@ No other variables than those in table and those in the standard:
     breakChar
     maxLookups
 
-Notice that versions 0, 1 & 2 of the table are supported. Notice also that the
+Notice that versions 0, 1, 2 & 3 of the table are supported. Notice also that the
 Panose variable has been broken down into its elements.
 
 =head1 METHODS
@@ -68,29 +68,81 @@ Panose variable has been broken down into its elements.
 =cut
 
 use strict;
-use vars qw(@ISA @fields @lens);
+use vars qw(@ISA @fields @lens @field_info @weights);
 use Font::TTF::Table;
+
 @ISA = qw(Font::TTF::Table);
+@field_info = (
+    'xAvgCharWidth' => 's',
+    'usWeightClass' => 'S',
+    'usWidthClass' => 'S',
+    'fsType' => 's',
+    'ySubscriptXSize' => 's',
+    'ySubScriptYSize' => 's',
+    'ySubscriptXOffset' => 's',
+    'ySubscriptYOffset' => 's',
+    'ySuperscriptXSize' => 's',
+    'ySuperscriptYSize' => 's',
+    'ySuperscriptXOffset' => 's',
+    'ySuperscriptYOffset' => 's',
+    'yStrikeoutSize' => 's',
+    'yStrikeoutPosition' => 's',
+    'sFamilyClass' => 's',
+    'bFamilyType' => 'C',
+    'bSerifStyle' => 'C',
+    'bWeight' => 'C',
+    'bProportion' => 'C',
+    'bContrast' => 'C',
+    'bStrokeVariation' => 'C',
+    'bArmStyle' => 'C',
+    'bLetterform' => 'C',
+    'bMidline' => 'C',
+    'bXheight' => 'C',
+    'ulUnicodeRange1' => 'L',
+    'ulUnicodeRange2' => 'L',
+    'ulUnicodeRange3' => 'L',
+    'ulUnicodeRange4' => 'L',
+    'achVendID' => 'L',
+    'fsSelection' => 'S',
+    'usFirstCharIndex' => 'S',
+    'usLastCharIndex' => 'S',
+    'sTypoAscender' => 'S',
+    'sTypoDescender' => 's',
+    'sTypoLineGap' => 'S',
+    'usWinAscent' => 'S',
+    'usWinDescent' => 'S',
+    '' => '',
+    'ulCodePageRange1' => 'L',
+    'ulCodePageRange2' => 'L',
+    '' => '',
+    'xHeight' => 's',
+    'CapHeight' => 's',
+    'defaultChar' => 'S',
+    'breakChar' => 'S',
+    'maxLookups' => 's',
+    '' => '',            # i.e. v3 is basically same as v2
+    );
+
+@weights = qw(64 14 27 35 100 20 14 42 63 3 6 35 20 56 56 17 4 49 56 71 31 10 18 3 18 2 166);
 
 use Font::TTF::Utils;
 
 sub init
 {
-    my ($k, $v, $c, $n, $i, $t);
+    my ($k, $v, $c, $n, $i, $t, $j);
 
     $n = 0;
-    @lens = (76, 84, 94);
-    while ($t = <Font::TTF::OS_2::DATA>)
+    @lens = (76, 84, 94, 94);
+    for ($j = 0; $j < $#field_info; $j += 2)
     {
-        $t =~ s/\r?\n$//;
-        if ($t =~ m/^\s*$/)
+        if ($field_info[$j] eq '')
         {
             $n++;
             next;
         }
-        ($k, $v, $c) = TTF_Init_Fields($t, $c);
+        ($k, $v, $c) = TTF_Init_Fields($field_info[$j], $c, $field_info[$j+1]);
         next unless defined $k && $k ne "";
-        for ($i = $n; $i < 3; $i++)
+        for ($i = $n; $i < 4; $i++)
         { $fields[$i]{$k} = $v; }
     }
 }
@@ -113,7 +165,7 @@ sub read
     $self->{' INFILE'}->read($dat, 2);
     $ver = unpack("n", $dat);
     $self->{'Version'} = $ver;
-    if ($ver < 3)
+    if ($ver < 4)
     {
         $self->{' INFILE'}->read($dat, $lens[$ver]);
         TTF_Read_Fields($self, $dat, $fields[$ver]);
@@ -200,24 +252,81 @@ be the negative of Descender.
 sub update
 {
     my ($self) = @_;
-    my ($map, @keys, $table);
+    my ($map, @keys, $table, $i, $avg, $hmtx);
 
     return undef unless ($self->SUPER::update);
 
     $self->{' PARENT'}{'cmap'}->update;
     $map = $self->{' PARENT'}{'cmap'}->find_ms || return undef;
+    $hmtx = $self->{' PARENT'}{'hmtx'}->read;
 
-    @keys = sort {$a <=> $b} keys %{$map->{'val'}};
+    @keys = sort {$a <=> $b} grep {$_ < 0x10000} keys %{$map->{'val'}};
 
     $self->{'usFirstCharIndex'} = $keys[0];
     $self->{'usLastCharIndex'} = $keys[-1];
 
-    $table = $self->{' PARENT'}{'hhea'};
-    $self->{'sTypoAscender'} = $table->{'Ascender'};
-    $self->{'sTypoDescender'} = $table->{'Descender'};
-    $self->{'sTypoLineGap'} = $table->{'Linegap'};
-    $self->{'usWinAscent'} = $self->{'sTypoAscender'} + $self->{'sTypoLineGap'};
-    $self->{'usWinDescent'} = -$self->{'sTypoDescender'};
+    $table = $self->{' PARENT'}{'hhea'}->read;
+    
+    # try any way we can to get some real numbers passed around!
+    if (($self->{'fsSelection'} & 128) != 0)
+    {
+        # assume the user knows what they are doing and has sensible values already
+    }
+    elsif ($table->{'Ascender'} != 0 || $table->{'Descender'} != 0)
+    {
+        $self->{'sTypoAscender'} = $table->{'Ascender'};
+        $self->{'sTypoDescender'} = $table->{'Descender'};
+        $self->{'sTypoLineGap'} = $table->{'LineGap'};
+        $self->{'usWinAscent'} = $self->{'sTypoAscender'} + $self->{'sTypoLineGap'};
+        $self->{'usWinDescent'} = -$self->{'sTypoDescender'};
+    }
+    elsif ($self->{'sTypoAscender'} != 0 || $self->{'sTypoDescender'} != 0)
+    {
+        $table->{'Ascender'} = $self->{'sTypoAscender'};
+        $table->{'Descender'} = $self->{'sTypoDescender'};
+        $table->{'LineGap'} = $self->{'sTypoLineGap'};
+        $self->{'usWinAscent'} = $self->{'sTypoAscender'} + $self->{'sTypoLineGap'};
+        $self->{'usWinDescent'} = -$self->{'sTypoDescender'};
+    } 
+    elsif ($self->{'usWinAscent'} != 0 || $self->{'usWinDescent'} != 0)
+    {
+        $self->{'sTypoAscender'} = $table->{'Ascender'} = $self->{'usWinAscent'};
+        $self->{'sTypoDescender'} = $table->{'Descender'} = -$self->{'usWinDescent'};
+        $self->{'sTypoLineGap'} = $table->{'LineGap'} = 0;
+    }
+
+    if ($self->{'Version'} < 3)
+    {
+        for ($i = 0; $i < 26; $i++)
+        { $avg += $hmtx->{'advance'}[$map->{'val'}{$i + 0x0061}] * $weights[$i]; }
+        $avg += $hmtx->{'advance'}[$map->{'val'}{0x0020}] * $weights[-1];
+        $self->{'xAvgCharWidth'} = $avg / 1000;
+    }
+    elsif ($self->{'Version'} > 2)
+    {
+        $i = 0; $avg = 0;
+        foreach (@{$hmtx->{'advance'}})
+        {
+            next unless ($_);
+            $i++;
+            $avg += $_;
+        }
+        $avg /= $i if ($i);
+        $self->{'xAvgCharWidth'} = $avg;
+    }
+
+    foreach $i (keys %{$map->{'val'}})
+    {
+        if ($i >= 0x10000)
+        {
+            $self->{'ulUnicodeRange2'} |= 0x2000000;
+            last;
+        }
+    }
+
+    $self->{'Version'} = 1 if (defined $self->{'ulCodePageRange1'} && $self->{'Version'} < 1);
+    $self->{'Version'} = 2 if (defined $self->{'maxLookups'} && $self->{'Version'} < 2);
+    
     $self;
 }
 
@@ -233,54 +342,3 @@ Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
 licensing.
 
 =cut
-
-
-__DATA__
-xAvgCharWidth, s
-usWeightClass, S
-usWidthClass, S
-fsType, s
-ySubscriptXSize, s
-ySubScriptYSize, s
-ySubscriptXOffset, s
-ySubscriptYOffset, s
-ySuperscriptXSize, s
-ySuperscriptYSize, s
-ySuperscriptXOffset, s
-ySuperscriptYOffset, s
-yStrikeoutSize, s
-yStrikeoutPosition, s
-sFamilyClass, s
-bFamilyType, C
-bSerifStyle, C
-bWeight, C
-bProportion, C
-bContrast, C
-bStrokeVariation, C
-bArmStyle, C
-bLetterform, C
-bMidline, C
-bXheight, C
-ulUnicodeRange1, L
-ulUnicodeRange2, L
-ulUnicodeRange3, L
-ulUnicodeRange4, L
-achVendID, L
-fsSelection, S
-usFirstCharIndex, S
-usLastCharIndex, S
-sTypoAscender, S
-sTypoDescender, s
-sTypoLineGap, S
-usWinAscent, S
-usWinDescent, S
-
-ulCodePageRange1, L
-ulCodePageRange2, L
-
-xHeight, s
-CapHeight, s
-defaultChar, S
-breakChar, S
-maxLookups, s
-
